@@ -3,34 +3,36 @@ use strict;
 use warnings;
 use lib './lib';
 use Class::STL::Containers;
+use Class::STL::Algorithms;
+use Class::STL::Utilities;
 
 print ">>>$0>>>:\n";
-my $v = Class::STL::Containers::Queue->new();
+my $v = queue();
 $v->push($v->factory(data => 'first'));
 $v->push($v->factory(data => 'second'));
 $v->push($v->factory(data => 'third'));
 $v->push($v->factory(data => 'fourth'));
 $v->push($v->factory(data => 'fifth'));
 
-$v->foreach(MyPrint->new('DATA:'));
-$v->back()->print(MyPrint->new('Back:'));
-$v->front()->print(MyPrint->new('Front:'));
+::foreach($v->begin(), $v->end(), MyPrint->new());
+print "Back:"; MyPrint->new()->function_operator($v->back());
+print "Front:"; MyPrint->new()->function_operator($v->front());
 print '$v->pop();', "\n";
 print '$v->push($v->factory(data => "sixth"));', "\n";
 $v->pop();
 $v->push($v->factory(data => 'sixth'));
-$v->back()->print(MyPrint->new('Back:'));
-$v->front()->print(MyPrint->new('Front:'));
+print "Back:"; MyPrint->new()->function_operator($v->back());
+print "Front:"; MyPrint->new()->function_operator($v->front());
 
 # ----------------------------------------------------------------------------------------------------
 {
 	package MyPrint;
-	use base qw(Class::STL::Utilities::UnaryFunction);
-	sub do
+	use base qw(Class::STL::Utilities::FunctionObject::UnaryFunction);
+	sub function_operator
 	{
 		my $self = shift;
-		my $elem = shift;
-		print $self->arg(), $elem->data(), "\n";
+		my $element = shift;
+		print "Data:", $element->data(), "\n";
 	}
 }
 # ----------------------------------------------------------------------------------------------------
